@@ -5,6 +5,7 @@ import itertools
 import os
 import random
 import shutil
+import signal
 import subprocess
 import sys
 import threading
@@ -265,9 +266,12 @@ class Cluster(object):
             binary = None
             if self.cassandra_version() >= '1.2':
                 binary = (ipformat % i, 9042)
+            thrift = None
+            if self.cassandra_version() < '4':
+                thrift = (ipformat % i, 9160)
             node = self.create_node(name='node%s' % i,
                                     auto_bootstrap=False,
-                                    thrift_interface=(ipformat % i, 9160),
+                                    thrift_interface=thrift,
                                     storage_interface=(ipformat % i, 7000),
                                     jmx_port=str(7000 + i * 100),
                                     remote_debug_port=str(2000 + i * 100) if debug else str(0),
@@ -361,7 +365,7 @@ class Cluster(object):
             else:
                 node.show(only_status=True)
 
-    def start(self, no_wait=False, verbose=False, wait_for_binary_proto=False,
+    def start(self, no_wait=False, verbose=False, wait_for_binary_proto=True,
               wait_other_notice=True, jvm_args=None, profile_options=None,
               quiet_start=False, allow_root=False):
         if jvm_args is None:
@@ -376,10 +380,7 @@ class Cluster(object):
             if not node.is_running():
                 for itf in node.network_interfaces.values():
                     if itf is not None:
-                        if not common.check_socket_available(itf, return_on_error=True):
-                            addr, port = itf
-                            common.error("Inet address {}:{} is not available; a cluster may already be running or you may need to add the loopback alias".format(addr, port))
-                            sys.exit(1)
+                        common.assert_socket_available(itf)
 
         started = []
         for node in list(self.nodes.values()):
@@ -428,11 +429,11 @@ class Cluster(object):
 
         return started
 
-    def stop(self, wait=True, gently=True):
+    def stop(self, wait=True, signal_event=signal.SIGTERM, **kwargs):
         not_running = []
         extension.pre_cluster_stop(self)
         for node in list(self.nodes.values()):
-            if not node.stop(wait, gently=gently):
+            if not node.stop(wait=wait, signal_event=signal_event, **kwargs):
                 not_running.append(node)
         extension.post_cluster_stop(self)
         return not_running
@@ -506,8 +507,7 @@ class Cluster(object):
 
     def set_configuration_options(self, values=None):
         if values is not None:
-            for k, v in iteritems(values):
-                self._config_options[k] = v
+            self._config_options = common.merge_configuration(self._config_options, values)
 
         self._persist_config()
         self.__update_topology_files()
@@ -685,3 +685,6 @@ class Cluster(object):
                     ret = namedtuple('Node_Log_Matching', 'node matchings')
                     return ret(node=node, matchings=matchings)
             time.sleep(1)
+
+    def wait_for_any_log(self, pattern, timeout, filename='system.log'):
+        return common.wait_for_any_log(self.nodelist(), pattern, timeout, filename=filename)
