@@ -2,6 +2,7 @@
 from __future__ import absolute_import
 
 import os
+import signal
 import subprocess
 import sys
 
@@ -227,10 +228,13 @@ class NodeStopCmd(Cmd):
         parser = self._get_default_parser(usage, self.description())
         parser.add_option('--no-wait', action="store_true", dest="no_wait",
                           help="Do not wait for the node to be stopped", default=False)
-        parser.add_option('-g', '--gently', action="store_true", dest="gently",
-                          help="Shut down gently (default)", default=True)
-        parser.add_option('--not-gently', action="store_false", dest="gently",
-                          help="Shut down immediately (kill -9)", default=True)
+        parser.add_option('-g', '--gently', action="store_const", dest="signal_event",
+                          help="Shut down gently (default)", const=signal.SIGTERM,
+                          default=signal.SIGTERM)
+        parser.add_option('--hang-up', action="store_const", dest="signal_event",
+                          help="Shut down via hang up (kill -1)", const=signal.SIGHUP)
+        parser.add_option('--not-gently', action="store_const", dest="signal_event",
+                          help="Shut down immediately (kill -9)", const=signal.SIGKILL)
         return parser
 
     def validate(self, parser, options, args):
@@ -238,7 +242,7 @@ class NodeStopCmd(Cmd):
 
     def run(self):
         try:
-            if not self.node.stop(not self.options.no_wait, gently=self.options.gently):
+            if not self.node.stop(wait=not self.options.no_wait, signal_event=self.options.signal_event):
                 print_("%s is not running" % self.name, file=sys.stderr)
                 exit(1)
         except NodeError as e:
@@ -330,8 +334,14 @@ class NodeDecommissionCmd(_NodeToolCmd):
     nodetool_cmd = 'decommission'
     descr_text = "Run decommission on node name"
 
+    def get_parser(self):
+        parser = self._get_default_parser(self.usage, self.description())
+        parser.add_option('--force', action="store_true", dest="force",
+                help="Force decommission of this node even when it reduces the number of replicas to below configured RF.  Note: This is only relevant for C* 3.12+.", default=False)
+        return parser
+
     def run(self):
-        self.node.decommission()
+        self.node.decommission(force=self.options.force)
 
 
 class _DseToolCmd(Cmd):
@@ -505,10 +515,9 @@ class NodeJsonCmd(Cmd):
                                            enumerate_keys=self.options.enumerate_keys)
             elif self.node.has_cmd('sstabledump'):
                 self.node.run_sstabledump(keyspace=self.keyspace,
-                                          output_file=f,
-                                          column_families=self.column_families,
-                                          keys=self.options.keys,
-                                          enumerate_keys=self.options.enumerate_keys)
+                                           column_families=self.column_families,
+                                           keys=self.options.keys,
+                                           enumerate_keys=self.options.enumerate_keys)
         except common.ArgumentError as e:
             print_(e, file=sys.stderr)
 
