@@ -16,7 +16,7 @@ import stat
 import subprocess
 import sys
 import time
-from distutils.version import LooseVersion  # pylint: disable=all
+from distutils.version import LooseVersion  #pylint: disable=import-error, no-name-in-module
 
 import yaml
 from six import print_
@@ -78,6 +78,10 @@ class ArgumentError(CCMError):
 class UnavailableSocketError(CCMError):
     pass
 
+class TimeoutError(Exception):
+
+    def __init__(self, data):
+        Exception.__init__(self, str(data))
 
 class LogPatternToVersion(object):
 
@@ -324,6 +328,20 @@ def is_win():
     return sys.platform in ("cygwin", "win32")
 
 
+def is_modern_windows_install(version):
+    """
+    The 2.1 release line was when Cassandra received beta windows support.
+    Many features are gated based on that added compatibility.
+
+    Handles floats, strings, and LooseVersions by first converting all three types to a string, then to a LooseVersion.
+    """
+    version = LooseVersion(str(version))
+    if is_win() and version >= LooseVersion('2.1'):
+        return True
+    else:
+        return False
+
+
 def is_ps_unrestricted():
     if not is_win():
         raise CCMError("Can only check PS Execution Policy on Windows")
@@ -461,7 +479,7 @@ def validate_install_dir(install_dir):
         raise ArgumentError('%s does not appear to be a cassandra or dse installation directory' % install_dir)
 
 
-def check_socket_available(itf, return_on_error=False):
+def assert_socket_available(itf):
     info = socket.getaddrinfo(itf[0], itf[1], socket.AF_UNSPEC, socket.SOCK_STREAM)
     if not info:
         raise UnavailableSocketError("Failed to get address info for [%s]:%s" % itf)
@@ -476,10 +494,8 @@ def check_socket_available(itf, return_on_error=False):
         return True
     except socket.error as msg:
         s.close()
-        if return_on_error:
-            return False
         addr, port = itf
-        raise UnavailableSocketError("Inet address %s:%s is not available: %s" % (addr, port, msg))
+        raise UnavailableSocketError("Inet address %s:%s is not available: %s; a cluster may already be running or you may need to add the loopback alias" % (addr, port, msg))
 
 
 def check_socket_listening(itf, timeout=60):
@@ -700,3 +716,27 @@ def is_intlike(obj):
     except TypeError:
         return False
     raise RuntimeError('Reached end of {}; should not be possible'.format(is_intlike.__name__))
+
+
+def wait_for_any_log(nodes, pattern, timeout, filename='system.log'):
+    """
+    Look for a pattern in the system.log of any in a given list
+    of nodes.
+    @param nodes The list of nodes whose logs to scan
+    @param pattern The target pattern
+    @param timeout How long to wait for the pattern. Note that
+                    strictly speaking, timeout is not really a timeout,
+                    but a maximum number of attempts. This implies that
+                    the all the grepping takes no time at all, so it is
+                    somewhat inaccurate, but probably close enough.
+    @return The first node in whose log the pattern was found
+    """
+    for _ in range(timeout):
+        for node in nodes:
+            found = node.grep_log(pattern, filename=filename)
+            if found:
+                return node
+        time.sleep(1)
+
+    raise TimeoutError(time.strftime("%d %b %Y %H:%M:%S", time.gmtime()) +
+                       " Unable to find: " + repr(pattern) + " in any node log within " + str(timeout) + "s")
