@@ -11,10 +11,11 @@ import stat
 import subprocess
 import sys
 import time
+import tempfile
 import warnings
 from collections import namedtuple
 from datetime import datetime
-from distutils.version import LooseVersion  # pylint: disable=all
+from distutils.version import LooseVersion  #pylint: disable=import-error, no-name-in-module
 
 import yaml
 from six import iteritems, print_, string_types
@@ -135,7 +136,10 @@ class Node(object):
             binary_interface = None
             if 'binary' in itf and itf['binary'] is not None:
                 binary_interface = tuple(itf['binary'])
-            node = cluster.create_node(data['name'], data['auto_bootstrap'], tuple(itf['thrift']), tuple(itf['storage']), data['jmx_port'], remote_debug_port, initial_token, save=False, binary_interface=binary_interface, byteman_port=data['byteman_port'])
+            thrift_interface = None
+            if 'thrift' in itf and itf['thrift'] is not None:
+                thrift_interface = tuple(itf['thrift'])
+            node = cluster.create_node(data['name'], data['auto_bootstrap'], thrift_interface, tuple(itf['storage']), data['jmx_port'], remote_debug_port, initial_token, save=False, binary_interface=binary_interface, byteman_port=data['byteman_port'])
             node.status = data['status']
             if 'pid' in data:
                 node.pid = int(data['pid'])
@@ -221,6 +225,10 @@ class Node(object):
         else:
             dir, v = setup(version, verbose=verbose)
             self.__install_dir = dir
+
+        if self.get_base_cassandra_version() >= '4':
+            self.network_interfaces['thrift'] = None
+
         self.import_config_files()
         self.import_bin_files()
         self.__conf_updated = False
@@ -248,9 +256,11 @@ class Node(object):
                 'concurrent_writes' : 64,
             })
         """
+        if not hasattr(self,'_config_options') or self.__config_options is None:
+            self.__config_options = {}
+
         if values is not None:
-            for k, v in iteritems(values):
-                self.__config_options[k] = v
+            self.__config_options = common.merge_configuration(self.__config_options, values)
 
         self.import_config_files()
 
@@ -291,7 +301,8 @@ class Node(object):
             if show_cluster:
                 print_("{}{}={}".format(indent, 'cluster', self.cluster.name))
             print_("{}{}={}".format(indent, 'auto_bootstrap', self.auto_bootstrap))
-            print_("{}{}={}".format(indent, 'thrift', self.network_interfaces['thrift']))
+            if self.network_interfaces['thrift'] is not None:
+                print_("{}{}={}".format(indent, 'thrift', self.network_interfaces['thrift']))
             if self.network_interfaces['binary'] is not None:
                 print_("{}{}={}".format(indent, 'binary', self.network_interfaces['binary']))
             print_("{}{}={}".format(indent, 'storage', self.network_interfaces['storage']))
@@ -384,10 +395,17 @@ class Node(object):
             return f.tell()
 
     def print_process_output(self, name, proc, verbose=False):
-        try:
-            stderr = proc.communicate()[1]
-        except ValueError:
-            stderr = ''
+        # If stderr_file exists on the process, we opted to
+        # store stderr in a separate temporary file, consume that.
+        if hasattr(proc, 'stderr_file') and proc.stderr_file is not None:
+            proc.stderr_file.seek(0)
+            stderr = proc.stderr_file.read()
+        else:
+            try:
+                stderr = proc.communicate()[1]
+            except ValueError:
+                stderr = ''
+
         if len(stderr) > 1:
             print_("[{} ERROR] {}".format(name, stderr.strip()))
 
@@ -408,12 +426,14 @@ class Node(object):
             return None
 
         log_file = os.path.join(self.get_path(), 'logs', filename)
+        output_read = False
         while not os.path.exists(log_file):
             time.sleep(.5)
-            if process:
+            if process and not output_read:
                 process.poll()
                 if process.returncode is not None:
                     self.print_process_output(self.name, process, verbose)
+                    output_read = True
                     if process.returncode != 0:
                         raise RuntimeError()  # Shouldn't reuse RuntimeError but I'm lazy
 
@@ -424,11 +444,12 @@ class Node(object):
             while True:
                 # First, if we have a process to check, then check it.
                 # Skip on Windows - stdout/stderr is cassandra.bat
-                if not common.is_win():
+                if not common.is_win() and not output_read:
                     if process:
                         process.poll()
                         if process.returncode is not None:
                             self.print_process_output(self.name, process, verbose)
+                            output_read = True
                             if process.returncode != 0:
                                 raise RuntimeError()  # Shouldn't reuse RuntimeError but I'm lazy
 
@@ -487,27 +508,30 @@ class Node(object):
         log for 'Starting listening for CQL clients' before checking for the
         interface to be listening.
 
-        Emits a warning if not listening after 10 seconds.
+        Emits a warning if not listening after 30 seconds.
         """
         if self.cluster.version() >= '1.2':
             self.watch_log_for("Starting listening for CQL clients", **kwargs)
 
         binary_itf = self.network_interfaces['binary']
-        if not common.check_socket_listening(binary_itf, timeout=10):
-            warnings.warn("Binary interface %s:%s is not listening after 10 seconds, node may have failed to start."
+        if not common.check_socket_listening(binary_itf, timeout=30):
+            warnings.warn("Binary interface %s:%s is not listening after 30 seconds, node may have failed to start."
                           % (binary_itf[0], binary_itf[1]))
 
     def wait_for_thrift_interface(self, **kwargs):
         """
         Waits for the Thrift interface to be listening.
 
-        Emits a warning if not listening after 10 seconds.
+        Emits a warning if not listening after 30 seconds.
         """
+        if self.cluster.version() >= '4':
+            return;
+
         self.watch_log_for("Listening for thrift clients...", **kwargs)
 
         thrift_itf = self.network_interfaces['thrift']
-        if not common.check_socket_listening(thrift_itf, timeout=10):
-            warnings.warn("Thrift interface {}:{} is not listening after 10 seconds, node may have failed to start.".format(thrift_itf[0], thrift_itf[1]))
+        if not common.check_socket_listening(thrift_itf, timeout=30):
+            warnings.warn("Thrift interface {}:{} is not listening after 30 seconds, node may have failed to start.".format(thrift_itf[0], thrift_itf[1]))
 
     def get_launch_bin(self):
         cdir = self.get_install_dir()
@@ -551,7 +575,7 @@ class Node(object):
             jvm_args += ['-Dcassandra.migration_task_wait_in_seconds={}'.format(len(self.cluster.nodes) * 2)]
 
         # Validate Windows env
-        if common.is_win() and not common.is_ps_unrestricted() and self.cluster.version() >= '2.1':
+        if common.is_modern_windows_install(self.cluster.version()) and not common.is_ps_unrestricted():
             raise NodeError("PS Execution Policy must be unrestricted when running C* 2.1+")
 
         if not common.is_win() and quiet_start:
@@ -562,7 +586,7 @@ class Node(object):
 
         for itf in list(self.network_interfaces.values()):
             if itf is not None and replace_address is None:
-                common.check_socket_available(itf)
+                common.assert_socket_available(itf)
 
         if wait_other_notice:
             marks = [(node, node.mark_log()) for node in list(self.cluster.nodes.values()) if node.is_live()]
@@ -621,6 +645,9 @@ class Node(object):
         process = None
         FNULL = open(os.devnull, 'w')
         stdout_sink = subprocess.PIPE if verbose else FNULL
+        # write stderr to a temporary file to prevent overwhelming pipe (> 65K data).
+        stderr = tempfile.SpooledTemporaryFile(max_size=0xFFFF)
+
         if common.is_win():
             # clean up any old dirty_pid files from prior runs
             if (os.path.isfile(self.get_path() + "/dirty_pid.tmp")):
@@ -629,12 +656,14 @@ class Node(object):
             if quiet_start and self.cluster.version() >= '2.2.4':
                 args.append('-q')
 
-            process = subprocess.Popen(args, cwd=self.get_bin_dir(), env=env, stdout=stdout_sink, stderr=subprocess.PIPE)
+            process = subprocess.Popen(args, cwd=self.get_bin_dir(), env=env, stdout=stdout_sink, stderr=stderr)
         else:
-            process = subprocess.Popen(args, env=env, stdout=stdout_sink, stderr=subprocess.PIPE)
+            process = subprocess.Popen(args, env=env, stdout=stdout_sink, stderr=stderr)
+
+        process.stderr_file = stderr
+
         # Our modified batch file writes a dirty output with more than just the pid - clean it to get in parity
         # with *nix operation here.
-
         if verbose:
             stdout, stderr = process.communicate()
             print_(stdout)
@@ -668,16 +697,23 @@ class Node(object):
 
         return process
 
-    def stop(self, wait=True, wait_other_notice=False, gently=True):
+    def stop(self, wait=True, wait_other_notice=False, signal_event=signal.SIGTERM, **kwargs):
         """
         Stop the node.
           - wait: if True (the default), wait for the Cassandra process to be
             really dead. Otherwise return after having sent the kill signal.
           - wait_other_notice: return only when the other live nodes of the
             cluster have marked this node has dead.
-          - gently: Let Cassandra clean up and shut down properly. Otherwise do
-            a 'kill -9' which shuts down faster.
+          - signal_event: Signal event to send to Cassandra; default is to
+            let Cassandra clean up and shut down properly (SIGTERM [15])
+          - Optional:
+             + gently: Let Cassandra clean up and shut down properly; unless
+                       false perform a 'kill -9' which shuts down faster.
         """
+        # Determine if the signal event should be updated to keep API compatibility
+        if 'gently' in kwargs and kwargs['gently'] is False:
+            signal_event = signal.SIGKILL
+
         if self.is_running():
             if wait_other_notice:
                 marks = [(node, node.mark_log()) for node in list(self.cluster.nodes.values()) if node.is_live() and node is not self]
@@ -692,7 +728,7 @@ class Node(object):
                 # We want the node to flush its data before shutdown as some tests rely on small writes being present.
                 # The default Periodic sync at 10 ms may not have flushed data yet, causing tests to fail.
                 # This is not a hard requirement, however, so we swallow any exceptions this may throw and kill anyway.
-                if gently is True:
+                if signal_event is signal.SIGTERM:
                     try:
                         self.flush()
                     except:
@@ -703,10 +739,7 @@ class Node(object):
                 if self._find_pid_on_windows():
                     common.warning("Failed to terminate node: {0} with pid: {1}".format(self.name, self.pid))
             else:
-                if gently:
-                    os.kill(self.pid, signal.SIGTERM)
-                else:
-                    os.kill(self.pid, signal.SIGKILL)
+                os.kill(self.pid, signal_event)
 
             if wait_other_notice:
                 for node, mark in marks:
@@ -833,11 +866,10 @@ class Node(object):
         cqlsh = self.get_tool('cqlsh')
         env = self.get_env()
         extension.append_to_client_env(self, env)
-        host = self.network_interfaces['thrift'][0]
         if self.get_base_cassandra_version() >= 2.1:
-            port = self.network_interfaces['binary'][1]
+            host, port = self.network_interfaces['binary']
         else:
-            port = self.network_interfaces['thrift'][1]
+            host, port = self.network_interfaces['thrift']
         args = []
         args += cqlsh_options
         extension.append_to_cqlsh_args(self, env, args)
@@ -911,20 +943,19 @@ class Node(object):
 
     def clear(self, clear_all=False, only_data=False):
         data_dirs = ['data{0}'.format(x) for x in xrange(0, self.cluster.data_dir_count)]
-        if not only_data:
-            data_dirs.append("commitlogs")
-            if clear_all:
-                data_dirs.extend(['saved_caches', 'logs'])
+        data_dirs.append("commitlogs")
+        if clear_all:
+            data_dirs.extend(['saved_caches', 'logs'])
         for d in data_dirs:
             full_dir = os.path.join(self.get_path(), d)
-            if only_data:
+            if only_data and d != "commitlogs":
                 for dir in os.listdir(full_dir):
                     keyspace_dir = os.path.join(full_dir, dir)
                     if os.path.isdir(keyspace_dir) and dir != "system":
                         for f in os.listdir(keyspace_dir):
-                            full_path = os.path.join(keyspace_dir, f)
-                            if os.path.isfile(full_path):
-                                os.remove(full_path)
+                            table_dir = os.path.join(keyspace_dir, f)
+                            shutil.rmtree(table_dir)
+                            os.mkdir(table_dir)
             else:
                 common.rmdirs(full_dir)
                 os.mkdir(full_dir)
@@ -1001,7 +1032,7 @@ class Node(object):
 
         return results
 
-    def run_sstablemetadata_process(self, output_file=None, datafiles=None, keyspace=None, column_families=None):
+    def run_sstablemetadata_process(self, datafiles=None, keyspace=None, column_families=None):
         cdir = self.get_install_dir()
         sstablemetadata = common.join_bin(cdir, os.path.join('tools', 'bin'), 'sstablemetadata')
         env = self.get_env()
@@ -1010,19 +1041,13 @@ class Node(object):
         cmd = [sstablemetadata]
         cmd.extend(sstablefiles)
 
-        if output_file is None:
-            return subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
-        else:
-            subprocess.call(cmd, env=env, stdout=output_file)
+        return subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
 
-    def run_sstablemetadata(self, output_file=None, datafiles=None, keyspace=None, column_families=None):
-        p = self.run_sstablemetadata_process(output_file, datafiles, keyspace, column_families)
-        if p is not None:
-            return handle_external_tool_process(p, "sstablemetadata on keyspace: {}, column_family: {}".format(keyspace, column_families))
-        else:
-            return None, None, None
+    def run_sstablemetadata(self, datafiles=None, keyspace=None, column_families=None):
+        p = self.run_sstablemetadata_process(datafiles, keyspace, column_families)
+        return handle_external_tool_process(p, "sstablemetadata on keyspace: {}, column_family: {}".format(keyspace, column_families))
 
-    def run_sstabledump_process(self, output_file=None, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
+    def run_sstabledump_process(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
         cdir = self.get_install_dir()
         sstabledump = common.join_bin(cdir, os.path.join('tools', 'bin'), 'sstabledump')
         env = self.get_env()
@@ -1039,19 +1064,16 @@ class Node(object):
             if keys is not None:
                 for key in keys:
                     cmd = cmd + ["-k", key]
-            if output_file is None:
-                p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
-                processes.append(p)
-            else:
-                subprocess.call(cmd, env=env, stdout=output_file)
+            p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
+            processes.append(p)
 
         for sstable in sstablefiles:
             do_dump(sstable)
 
         return processes
 
-    def run_sstabledump(self, output_file=None, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
-        processes = self.run_sstabledump_process(output_file, datafiles, keyspace, column_families, keys, enumerate_keys)
+    def run_sstabledump(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
+        processes = self.run_sstabledump_process(datafiles, keyspace, column_families, keys, enumerate_keys)
         results = []
 
         for p in processes:
@@ -1059,37 +1081,30 @@ class Node(object):
 
         return results
 
-    def run_sstableexpiredblockers_process(self, output_file=None, keyspace=None, column_family=None):
+    def run_sstableexpiredblockers_process(self, keyspace=None, column_family=None):
         cdir = self.get_install_dir()
         sstableexpiredblockers = common.join_bin(cdir, os.path.join('tools', 'bin'), 'sstableexpiredblockers')
         env = self.get_env()
         cmd = [sstableexpiredblockers, keyspace, column_family]
-        if output_file is None:
-            return subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
-        else:
-            subprocess.call(cmd, env=env, stdout=output_file)
+        return subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
 
-    def run_sstableexpiredblockers(self, output_file=None, keyspace=None, column_family=None):
-        p = self.run_sstableexpiredblockers_process(output_file=output_file, keyspace=keyspace, column_family=column_family)
+    def run_sstableexpiredblockers(self, keyspace=None, column_family=None):
+        p = self.run_sstableexpiredblockers_process(keyspace=keyspace, column_family=column_family)
         if p is not None:
             return handle_external_tool_process(p, ['sstableexpiredblockers', keyspace, column_family])
         else:
             return None, None, None
 
-    def run_sstableupgrade_process(self, output_file=None, keyspace=None, column_family=None):
+    def run_sstableupgrade_process(self, keyspace=None, column_family=None):
         cdir = self.get_install_dir()
         sstableupgrade = self.get_tool('sstableupgrade')
         env = self.get_env()
         cmd = [sstableupgrade, keyspace, column_family]
-        if output_file is None:
-            p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
-            return p
-        else:
-            subprocess.call(cmd, env=env, stdout=output_file)
-            return None
+        p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
+        return p
 
-    def run_sstableupgrade(self, output_file=None, keyspace=None, column_family=None):
-        p = self.run_sstableupgrade_process(output_file, keyspace, column_family)
+    def run_sstableupgrade(self, keyspace=None, column_family=None):
+        p = self.run_sstableupgrade_process(keyspace, column_family)
         if p is not None:
             return handle_external_tool_process(p, "sstableupgrade on {} : {}".format(keyspace, column_family))
         else:
@@ -1301,8 +1316,11 @@ class Node(object):
     def cleanup(self):
         self.nodetool("cleanup")
 
-    def decommission(self):
-        self.nodetool("decommission")
+    def decommission(self, force=False):
+        cmd = 'decommission'
+        if force:
+            cmd += " --force"
+        self.nodetool(cmd)
         self.status = Status.DECOMMISSIONED
         self._update_config()
 
@@ -1450,9 +1468,10 @@ class Node(object):
             # cassandra 0.8
             data['seed_provider'][0]['parameters'][0]['seeds'] = ','.join(self.cluster.get_seeds())
         data['listen_address'], data['storage_port'] = self.network_interfaces['storage']
-        data['rpc_address'], data['rpc_port'] = self.network_interfaces['thrift']
+        if self.network_interfaces['thrift'] is not None and self.get_base_cassandra_version() < 4:
+            data['rpc_address'], data['rpc_port'] = self.network_interfaces['thrift']
         if self.network_interfaces['binary'] is not None and self.get_base_cassandra_version() >= 1.2:
-            _, data['native_transport_port'] = self.network_interfaces['binary']
+            data['rpc_address'], data['native_transport_port'] = self.network_interfaces['binary']
 
         data['data_file_directories'] = [os.path.join(self.get_path(), 'data{0}'.format(x)) for x in xrange(0, self.cluster.data_dir_count)]
         data['commitlog_directory'] = os.path.join(self.get_path(), 'commitlogs')
@@ -1547,7 +1566,7 @@ class Node(object):
         agentlib_setting = '-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address={}'.format(str(self.remote_debug_port))
         remote_debug_options = agentlib_setting
         # The cassandra-env.ps1 file has been introduced in 2.1
-        if common.is_win() and self.get_base_cassandra_version() >= 2.1:
+        if common.is_modern_windows_install(self.get_base_cassandra_version()):
             conf_file = os.path.join(self.get_conf_dir(), common.CASSANDRA_WIN_ENV)
             jvm_file = os.path.join(self.get_conf_dir(), common.JVM_OPTS)
             jmx_port_pattern = '^\s+\$JMX_PORT='
@@ -1564,7 +1583,7 @@ class Node(object):
 
         common.replace_in_file(conf_file, jmx_port_pattern, jmx_port_setting)
 
-        if common.is_win() and common.get_version_from_build(node_path=self.get_path()) >= '2.1':
+        if common.is_modern_windows_install(common.get_version_from_build(node_path=self.get_path())):
             dst = os.path.join(self.get_conf_dir(), common.CASSANDRA_WIN_ENV)
             replacements = [
                 ('env:CASSANDRA_HOME =', '        $env:CASSANDRA_HOME="%s"' % self.get_install_dir()),
@@ -1583,7 +1602,7 @@ class Node(object):
         if self.byteman_port != '0':
             byteman_jar = glob.glob(os.path.join(self.get_install_dir(), 'build', 'lib', 'jars', 'byteman-[0-9]*.jar'))[0]
             agent_string = "-javaagent:{}=listener:true,boot:{},port:{}".format(byteman_jar, byteman_jar, str(self.byteman_port))
-            if common.is_win() and self.get_base_cassandra_version() >= 2.1:
+            if common.is_modern_windows_install(self.get_base_cassandra_version()):
                 with open(conf_file, "r+") as conf_rewrite:
                     conf_lines = conf_rewrite.readlines()
                     # Remove trailing brace, will be replaced
@@ -1772,7 +1791,7 @@ class Node(object):
 
         try:
             with open(pidfile, 'rb') as f:
-                if common.is_win() and self.get_base_cassandra_version() >= 2.1:
+                if common.is_modern_windows_install(self.get_base_cassandra_version()):
                     self.pid = int(f.readline().strip().decode('utf-16').strip())
                 else:
                     self.pid = int(f.readline().strip())
