@@ -11,10 +11,15 @@ import sys
 import tarfile
 import tempfile
 import time
-from distutils.version import LooseVersion  # pylint: disable=all
+from distutils.version import LooseVersion  #pylint: disable=import-error, no-name-in-module
 
 from six import next, print_
 
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
+
 from ccmlib import common
 from ccmlib.common import (ArgumentError, CCMError,
                            assert_jdk_valid_for_cassandra_version,
@@ -27,6 +32,8 @@ OPSC_ARCHIVE = "http://downloads.datastax.com/community/opscenter-%s.tar.gz"
 ARCHIVE = "http://archive.apache.org/dist/cassandra"
 GIT_REPO = "http://git-wip-us.apache.org/repos/asf/cassandra.git"
 GITHUB_TAGS = "https://api.github.com/repos/apache/cassandra/git/refs/tags"
+CCM_CONFIG = ConfigParser.ConfigParser()
+CCM_CONFIG.read(os.path.join(os.path.expanduser("~"), ".ccm", "config"))
 
 
 def setup(version, verbose=False):
@@ -64,6 +71,16 @@ def setup(version, verbose=False):
         binary = False
         fallback = False
 
+    elif version.startswith('alias:'):
+        alias = version.split(":")[1].split("/")[0]
+        try:
+            git_repo = CCM_CONFIG.get("aliases", alias)
+            clone_development(git_repo, version, verbose=verbose, alias=True)
+            return (directory_name(version), None)
+        except ConfigParser.NoOptionError as e:
+            common.warning("Unable to find alias {} in configuration file.".format(alias))
+            raise e
+
     if version in ('stable', 'oldstable', 'testing'):
         version = get_tagged_version_numbers(version)[0]
 
@@ -111,7 +128,7 @@ def validate(path):
         setup(version)
 
 
-def clone_development(git_repo, version, verbose=False):
+def clone_development(git_repo, version, verbose=False, alias=False):
     print_(git_repo, version)
     target_dir = directory_name(version)
     assert target_dir
@@ -120,6 +137,9 @@ def clone_development(git_repo, version, verbose=False):
     elif 'local:' in version:
         git_repo_name = 'local_{}'.format(git_repo)  # add git repo location to distinguish cache location for differing repos
         git_branch = version.split(':')[-1]  # last token on 'local:...' slugs should always be branch name
+    elif alias:
+        git_repo_name = 'alias_{}'.format(version.split('/')[0].split(':')[-1])
+        git_branch = version.split('/')[-1]
     else:
         git_repo_name = 'apache'
         git_branch = version.split(':', 1)[1]
@@ -244,7 +264,7 @@ def download_opscenter_version(version, target_version, verbose=False):
         __download(url, target, show_progress=verbose)
         common.info("Extracting {} as version {} ...".format(target, target_version))
         tar = tarfile.open(target)
-        dir = next(tar).name.split("/")[0]
+        dir = tar.next().name.split("/")[0]  # pylint: disable=all
         tar.extractall(path=__get_dir())
         tar.close()
         target_dir = os.path.join(__get_dir(), target_version)
@@ -275,7 +295,7 @@ def download_version(version, url=None, verbose=False, binary=False):
         __download(u, target, show_progress=verbose)
         common.info("Extracting {} as version {} ...".format(target, version))
         tar = tarfile.open(target)
-        dir = next(tar).name.split("/")[0]
+        dir = tar.next().name.split("/")[0]  # pylint: disable=all
         tar.extractall(path=__get_dir())
         tar.close()
         target_dir = os.path.join(__get_dir(), version)
